@@ -1,7 +1,6 @@
-/* $Id: cpl_image_iqe.c,v 1.33 2013-01-16 16:45:08 llundin Exp $
- *
+/*
  * This file is part of the ESO Common Pipeline Library
- * Copyright (C) 2001-2008 European Southern Observatory
+ * Copyright (C) 2001-2014 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +17,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-/*
- * $Author: llundin $
- * $Date: 2013-01-16 16:45:08 $
- * $Revision: 1.33 $
- * $Name: not supported by cvs2svn $
- */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -36,12 +28,16 @@
 #include "cpl_image_iqe.h"
 
 #include "cpl_error_impl.h"
+#include "cpl_memory.h"
+#include "cpl_tools.h"
 #include "cpl_bivector.h"
 #include "cpl_image_basic.h"
 #include "cpl_image_defs.h"
 #include "cpl_math_const.h"
+#include "cpl_mpfit.h"
 
 #include <math.h>
+#include <string.h>
 
 /*-----------------------------------------------------------------------------
                             Private Function prototypes
@@ -171,19 +167,11 @@ cpl_bivector * cpl_image_iqe(const cpl_image * in,
 /* Declare local functions private to avoid namespace conflicts */
 /* Also add function declaration, to prevent problems in case the ieq() code
    is copied once again from RTD. */
-static int gaussj(double *, int, double *, int);
-static int covsrt(double *, int, int [], int);
 static int estm9p(float *, int, int, int, int, float *, float *, float *);
 static int iqebgv(float *, int, int, float *, float *, int *);
 static int iqemnt(float *, int, int, float, float, float *, int);
 static int iqesec(float *, int, int, float, float *, float *, int);
 static int iqefit(float *, int, int, float, float *, float *, float *, int);
-static void indexx(int, float [], int []);
-static void hsort(int, float *);
-static int mrqmin(int, float [], int, int [], int, double *, double *, double *,
-                  int (*)(), double *);
-static int mrqcof(int, float [], int, int [], int, double *, double [],
-                  double *, int (*)());
 static int g2efit(float *, float *, int, int, float [], float [], double *);
 static int g2einit(float *, float *, int, int);
 static int g2efunc(int, float *, float *, float *, float *, float *);
@@ -314,7 +302,7 @@ int        *nbg
 
   ns = (mx<my) ? mx - 1 : my - 1;
   ms  = (mx<my) ? mx/4 : my/4;
-  pfb = (float *) calloc(8*ns*ms, sizeof(float));
+  pfb = (float *) cpl_calloc(8*ns*ms, sizeof(float));
   if (!pfb) return -1;
   pwb = pfb + 4*ns*ms;
 
@@ -344,7 +332,7 @@ int        *nbg
   n = nt;
      mt = nt;
      while (n--) *pw++ = 1.0;
-  hsort(mt, pfb);
+  cpl_tools_sort_ascn_float(pfb, mt);
   nt = mt;
 
 /* first estimate of mean and rms   */
@@ -368,7 +356,7 @@ int        *nbg
      }
     else *pw = 0.0;
       }
-     if (mt<1) { free (pfb); return -2; }
+     if (mt<1) { cpl_free (pfb); return -2; }
      ba = bm/mt; bs = bs/mt - ba*ba;
      bs = (0.0<bs) ? sqrt(bs) : 0.0;
    }
@@ -378,7 +366,7 @@ int        *nbg
   *bgm = ba;
   *bgs = bs;
   *nbg = mt;
-  free(pfb);
+  cpl_free(pfb);
 
   return 0;
 }
@@ -557,9 +545,11 @@ float      *dx,
 float      *dy
 )
 {
-int      n, nt, ix, iy, idx[9];
+int      n, nt, ix, iy;
 float    a, am;
-float    *pfb, *pwb, fb[9], wb[9];
+float    *pfb, *pwb, fb[9], _fb[9], wb[9];
+
+cpl_size  idx[9];
 
 /* check if 3x3 region is fully within frame   */
 
@@ -581,8 +571,8 @@ pfm += nx-1 + mx*(ny-1);
          }
       pfm += mx - 3;
       }
-indexx(9, fb, idx);
-
+memcpy(_fb, fb, 9 * sizeof(float));
+cpl_tools_sort_stable_pattern_float(_fb, 9, 0, 0, idx);
 /* omit largest value and estimate mean     */
 
 wb[idx[8]] = 0.0;
@@ -744,7 +734,7 @@ if (mx<nxs+nx) nx = mx - nxs;
 ny = ceil(8.0*est[4]);
 if (my<nys+ny) ny = my - nys;
 
-pfb = (float *) calloc(2*nx*ny, sizeof(float));
+pfb = (float *) cpl_calloc(2*nx*ny, sizeof(float));
 if (!pfb) return -10;
 pwb = pfb + nx*ny;
 
@@ -753,7 +743,7 @@ pwb = pfb + nx*ny;
 
 pfm += nxs + mx*nys;
 psize = pfm - pfmo;
-if ((psize < 0) || (psize > winsize)) { free (pfb); return -99; }
+if ((psize < 0) || (psize > winsize)) { cpl_free (pfb); return -99; }
 
 
 pf = pfb; pw = pwb;
@@ -765,13 +755,13 @@ iy = ny;
          {
          *pf++ = *pfm++ - bgv;
          psize = pfm - pfmo;
-         if (psize > winsize) { free (pfb); return -99; }
+         if (psize > winsize) { cpl_free (pfb); return -99; }
 
          *pw++ = 1.0;
          }
       pfm += mx - nx;
       psize = pfm - pfmo;
-      if ((psize < 0) || (psize > winsize)) { free (pfb); return -99; }
+      if ((psize < 0) || (psize > winsize)) { cpl_free (pfb); return -99; }
       }
 
 /* initialize parameters for fitting    */
@@ -792,7 +782,7 @@ n = g2efit(pfb, pwb, nx, ny, ap, cm, &chi);
 ap[1] += nxs;
 ap[3] += nys;
 
-free(pfb);
+cpl_free(pfb);
 return n;
 }
 
@@ -892,6 +882,54 @@ float      *dyda
   return 0;
 }
 
+static int g2efunc2(int ndata, int npar, double *p, double *deviates,
+                    double **derivs, void *d)
+{
+    int i, j;
+    float * dyda = cpl_malloc(npar * sizeof(*dyda));
+    float fp[MA];
+
+    for (j = 0; j < MA; j++) {
+        fp[j] = p[j];
+    }
+
+    /* Compute function deviates */
+    for (i=0; i<ndata; i++) {
+        float z, err, val;
+        int st = g2efunc(i, &val, &z, &err, fp, dyda);
+        if (st < 0) {
+            /* error */
+            cpl_free(dyda);
+            return st;
+        }
+        if (st > 0 || err == 0.) {
+            /* bad pixel */
+            deviates[i] = 0.;
+            if (derivs) {
+                int k;
+                for (k=0; k<npar; k++) {
+                    if (derivs[k]) {
+                        derivs[k][i] = 0.;
+                    }
+                }
+            }
+        }
+        else {
+            deviates[i] = (val - z) / err;
+            if (derivs) {
+                int k;
+                for (k=0; k<npar; k++) {
+                    if (derivs[k]) {
+                        derivs[k][i] = -dyda[k] / err;
+                    }
+                }
+            }
+        }
+    }
+    cpl_free(dyda);
+    return 0;
+}
+
 static int g2efit(
 /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 .PURPOSE   Perform 2D Gauss fit
@@ -907,363 +945,44 @@ float      cv[MA],
 double     *pchi
 )
 {
-  int      mt, n, na, ni, lista[MA];
-  float    apo[MA];
-  double   c2, a1, a2, alpha[MA*MA], cvm[MA*MA];
+    int i;
+    int status;
+    double * a = cpl_malloc((MA) * sizeof(*a));
+    mp_par * pars = cpl_calloc(MA, sizeof(*pars));
+    mp_result result;
 
-  if (g2einit(val, wght, nx, ny)) return -1;
+    if (g2einit(val, wght, nx, ny)) return -1;
 
-  a1 = -1.0;
-  mt = nx * ny;
-  for (n=0; n<MA; n++) { lista[n] = n; cv[n] = 0.0; }
-
-  *pchi = c2 = 0.0; a2 = 0.0; na = 0;
-  for (ni=0; ni<MITER; ni++) {
-     for (n=0; n<MA; n++) apo[n] = ap[n];
-     if (mrqmin(mt, ap, MA, lista, MA, cvm, alpha, pchi, g2efunc, &a1))
-       return -2;
-     if (a1<a2 && fabs(*pchi-c2)<1.0e-5*c2) break;
-     if (a1<a2) { c2 = *pchi; na = 0; } else na++;
-     a2 = a1;
-     if (5<na) break;
-     if (ap[0]<=0.0) ap[0] = 0.5 * apo[0];
-     if (ap[2]<=0.0) ap[2] = 0.5 * apo[2];
-     if (ap[4]<=0.0) ap[4] = 0.5 * apo[4];
-     ap[5] = fmod(ap[5], CPL_MATH_PI);
-     if (ap[1]<0.0 || nx<ap[1] || ap[3]<0.0 || ny<ap[3]) return -3;
-   }
-
-  a1 = 0.0;
-  if (mrqmin(mt, ap, MA, lista, MA, cvm, alpha, pchi, g2efunc, &a1))
-    return -2;
-
-  ap[5] = fmod(ap[5]+CPL_MATH_PI, CPL_MATH_PI);
-  for (n=0; n<MA; n++) cv[n] = sqrt(cvm[n+n*MA]);
-
-  return ((MITER<=ni) ? -4 : ni);
-}
-
-static      float          atry[MMA];
-static      double           da[MMA];
-static      double        oneda[MMA];
-static      double         beta[MMA];
-static      double       cv[MMA*MMA];
-static      double            ochisq;
-
-#ifdef _OPENMP
-#pragma omp threadprivate(atry,da,oneda,beta,cv,ochisq)
-#endif
-
-
-static int mrqmin(
-/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-.PURPOSE
-.RETURN    status,  0: OK, -1: Bad permutation LISTA 1,
-                   -2: Bad permutation LISTA 2, -3: too many variables,
-                   -4: No points (chisq<=0), -5: error in matrix inversion
-------------------------------------------------------------------------*/
-int       ndata,
-float     a[],
-int       ma,
-int       lista[],
-int       mfit,
-double    *covar,
-double    *alpha,
-double    *chisq,
-int       (*funcs)(int, float *, float *, float *, float *, float *),
-double    *alamda
-)
-{
-  int     k, kk, j, ihit;
-
-  if (*alamda < 0.0) {
-     if (MMA<ma || ma<mfit) return -3;
-     kk = mfit;
-     for (j=0; j<ma; j++) {
-    ihit = 0;
-    for (k=0; k<mfit; k++)
-      if (lista[k] == j) ihit++;
-    if (ihit == 0)
-      lista[kk++] = j;
-    else if (ihit > 1) return -1;
-      }
-     if (kk != ma) return -2;
-     *alamda = 0.001;
-     mrqcof(ndata, a, ma, lista, mfit, alpha, beta, chisq, funcs);
-     if (*chisq<=0.0) return -4;
-     ochisq = (*chisq);
-   }
-
-  for (j=0; j<mfit; j++) {
-     for (k=0; k<mfit; k++) covar[j+k*ma] = cv[j+k*mfit] = alpha[j+k*ma];
-     covar[j+j*ma] = cv[j+j*mfit] = alpha[j+j*ma]*(1.0+(*alamda));
-     oneda[j] = beta[j];
-   }
-
-  if (gaussj(cv, mfit, oneda, 1)) return -5;
-  for (j=0; j<mfit; j++) da[j] = oneda[j];
-
-  if (*alamda == 0.0) {
-     for (j=0; j<mfit; j++)
-       for (k=0; k<mfit; k++) covar[j+k*ma] = cv[j+k*mfit];
-     covsrt(covar, ma, lista, mfit);
-     return 0;
-   }
-
-  for (j=0; j<ma; j++) atry[j] = a[j];
-  for (j=0; j<mfit; j++)
-    atry[lista[j]] = a[lista[j]]+da[j];
-
-  mrqcof(ndata, atry, ma, lista, mfit, covar, da, chisq, funcs);
-  if (0.0<*chisq && *chisq<ochisq) {
-     *alamda *= 0.1;
-     ochisq = (*chisq);
-     for (j=0; j<mfit; j++) {
-    for (k=0; k<mfit; k++) alpha[j+k*ma] = covar[j+k*ma];
-    beta[j] = da[j];
-    a[lista[j]] = atry[lista[j]];
-      }
-  } else {
-     *alamda *= 10.0;
-     *chisq = ochisq;
-   }
-
-  return 0;
-}
-
-static int mrqcof(
-/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-.PURPOSE   compute covarient matrix and chisq from all values
-.RETURN    always 0
-------------------------------------------------------------------------*/
-int       ndata,
-float     a[],
-int       ma,
-int       lista[],
-int       mfit,
-double    *alpha,
-double    veta[],
-double    *chisq,
-int       (*funcs)(int, float *, float *, float *, float *, float *)
-)
-{
-  int     k, j, i;
-  float   ymod, wt, sig2i, dy, y;
-  float   dyda[MMA];
-
-  for (j=0; j<mfit; j++) {
-     for (k=0; k<=j; k++) alpha[j+k*ma] = 0.0;
-     veta[j] = 0.0;
-   }
-
-  *chisq = 0.0;
-  for (i=0; i<ndata; i++) {
-     if ((*funcs)(i, &y, &ymod, &sig2i, a, dyda)) continue;
-     dy = y - ymod;
-     for (j=0; j<mfit; j++) {
-    wt = dyda[lista[j]]*sig2i;
-    for (k=0; k<=j; k++)
-      alpha[j+k*ma] += wt*dyda[lista[k]];
-    veta[j] += dy*wt;
-      }
-     (*chisq) += dy*dy*sig2i;
-   }
-
-  for (j=1; j<mfit; j++)
-    for (k=0; k<j; k++)
-       alpha[k+j*ma] = alpha[j+k*ma];
-
-  return 0;
-}
-
-static int gaussj(
-/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-.PURPOSE   Inverse matrix using Gauss-Jordan elimination
-.RETURN    status,  0: OK, -1: Singular matrix 1, -2: Singular matrix 2,
-                   -3: matrix too big
-------------------------------------------------------------------------*/
-double     *a,
-int        n,
-double     *b,
-int        m
-)
-{
-  int      i, icol, irow, j, k, l, ll;
-  int      indxc[MMA], indxr[MMA], ipiv[MMA];
-  double   big, dum, pivinv;
-
-  /* Missing initialization fixed */
-  icol = irow = 0;
-
-  if (MMA<n) return -3;
-  for (j=0; j<n; j++) ipiv[j]=0;
-
-  for (i=0; i<n; i++) {
-     big = 0.0;
-     for (j=0; j<n; j++)
-       if (ipiv[j] != 1) {
-      for (k=0; k<n; k++) {
-         if (ipiv[k] == 0) {
-        dum = fabs(a[j+k*n]);
-        if (dum >= big) {
-           big = dum;
-           irow = j;
-           icol = k;
-         }
-         } else if (ipiv[k] > 1) return -1;
-       }
+    memset(&result, 0, sizeof(result));
+    for (i = 0; i < MA; i++) {
+        a[i] = ap[i];
+        pars[i].side = 3;
     }
+    /* no negative sigma */
+    pars[2].limited[0] = 1;
+    pars[2].limits[0] = 0.;
+    pars[4].limited[0] = 1;
+    pars[4].limits[0] = 0.;
 
-     ++(ipiv[icol]);
-     if (irow != icol) {
-    for (l=0; l<n; l++) SWAP(a[irow+l*n],a[icol+l*n])
-      for (l=0; l<m; l++) SWAP(b[irow+l*n],b[icol+l*n])
-      }
+    result.xerror = cpl_malloc(MA * sizeof(result.xerror[0]));
 
-     indxr[i] = irow;
-     indxc[i] = icol;
-     if (a[icol+icol*n] == 0.0) return -2;
-     pivinv = 1.0/a[icol+icol*n];
-     a[icol+icol*n] = 1.0;
-     for (l=0; l<n; l++) a[icol+l*n] *= pivinv;
-     for (l=0; l<m; l++) b[icol+l*n] *= pivinv;
-     for (ll=0; ll<n; ll++)
-       if (ll != icol) {
-      dum = a[ll+icol*n];
-      a[ll+icol*n] = 0.0;
-      for (l=0; l<n; l++) a[ll+l*n] -= a[icol+l*n]*dum;
-      for (l=0; l<m; l++) b[ll+l*n] -= b[icol+l*n]*dum;
+    status = mpfit((mp_func)&g2efunc2, nx * ny, MA,
+                   a, pars, NULL, NULL, &result);
+
+
+    for (i = 0; i < MA; i++) {
+        ap[i] = a[i];
+        cv[i] = result.xerror[i];
     }
-   }
+    ap[5] = fmod(ap[5], 4.0*atan(1.0));
+    *pchi = result.bestnorm;
 
-  for (l=n-1; l>=0; l--) {
-     if (indxr[l] != indxc[l])
-       for (k=0; k<n; k++)
-     SWAP(a[k+indxr[l]*n],a[k+indxc[l]*n]);
-   }
-
-  return 0;
-}
-
-static int covsrt(
-/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-.PURPOSE   compute covariance matrix
-.RETURN    status, always 0: OK
-------------------------------------------------------------------------*/
-double     *covar,
-int        ma,
-int        lista[],
-int        mfit
-)
-{
-  int      i, j;
-  double   swap;
-
-  for (j=0; j<ma-1; j++)
-    for (i=j+1; i<ma; i++) covar[i+j*ma] = 0.0;
-
-  for (i=0; i<mfit-1; i++)
-    for (j=i+1; j<mfit; j++) {
-       if (lista[j] > lista[i])
-     covar[lista[j]+lista[i]*ma] = covar[i+j*ma];
-       else
-     covar[lista[i]+lista[j]*ma] = covar[i+j*ma];
-     }
-
-  swap = covar[0];
-  for (j=0; j<ma; j++) {
-     covar[j*ma] = covar[j+j*ma];
-     covar[j+j*ma] = 0.0;
-   }
-
-  covar[lista[0]+lista[0]*ma] = swap;
-  for (j=1; j<mfit; j++) covar[lista[j]+lista[j]*ma] = covar[j*ma];
-  for (j=1; j<ma; j++)
-    for (i=0; i<j; i++) covar[i+j*ma] = covar[j+i*ma];
-
-  return 0;
-}
-
-static void indexx(
-/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-.PURPOSE   compute indx[] so that arrin[indx[0..n]] is ascenting
-.RETURN    none
-----------------------------------------------------------------------*/
-int     n,
-float   arrin[],
-int     indx[]
-)
-{
-  int     l, j, ir, indxt, i;
-  float   q;
-
-  for (j=0; j<n; j++) indx[j] = j;
-  l =  n >> 1;
-  ir = n - 1;
-  while(1) {
-     if (l>0) {
-    indxt = indx[--l];
-    q = arrin[indxt];
-      }
-     else {
-    indxt = indx[ir];
-    q = arrin[indxt];
-    indx[ir] = indx[0];
-    if (--ir == 0) {
-       indx[0] = indxt;
-       return;
-     }
-      }
-     i = l;
-     j = (l<<1) + 1;
-     while (j<=ir) {
-    if (j<ir && arrin[indx[j]]<arrin[indx[j+1]]) j++;
-    if (q<arrin[indx[j]]) {
-       indx[i] = indx[j];
-       j += (i=j) + 1;
-     }
-    else break;
-      }
-     indx[i] = indxt;
-   }
-}
-
-static void hsort(
-/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-.PURPOSE   sort array in place using heapsort
-.RETURN    none
------------------------------------------------------------------------*/
-int        n,                    /* no. of elements in array           */
-float      *ra                   /* pointer to array to be sorted      */
-)
-{
-  int      l, j, ir, i;
-  float    rra;
-
-  l = n >> 1;
-  ir = n - 1;
-
-  while (1) {
-     if (l>0)
-       rra = ra[--l];
-     else {
-    rra = ra[ir];
-    ra[ir] = ra[0];
-    if (--ir == 0) {
-       ra[0] = rra;
-       return;
-     }
-      }
-     i = l;
-     j = (l << 1) + 1;
-     while (j<=ir) {
-    if (j<ir && ra[j]<ra[j+1]) ++j;
-    if (rra<ra[j]) {
-       ra[i] = ra[j];
-       j += (i=j) + 1;
-     }
-    else j = ir + 1;
-      }
-     ra[i] = rra;
-   }
+    cpl_free(a);
+    cpl_free(result.xerror);
+    cpl_free(pars);
+    
+    if (status <= 0) return -2;
+    if (ap[1]<0.0 || nx<ap[1] || ap[3]<0.0 || ny<ap[3]) return -3;
+    if (result.niter > MITER) return -4;
+    return result.niter;
 }
