@@ -1,4 +1,4 @@
-/* $Id: cpl_math_const.h,v 1.10 2012/10/01 14:50:02 llundin Exp $
+/* $Id: cpl_math_const.h,v 1.10 2012-10-01 14:50:02 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/10/01 14:50:02 $
+ * $Date: 2012-10-01 14:50:02 $
  * $Revision: 1.10 $
- * $Name: cpl-6_3_1 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifndef CPL_MATH_CONST_H
@@ -271,6 +271,36 @@ CPL_BEGIN_DECLS
 /*----------------------------------------------------------------------------*/
 #define CPL_MATH_STD_MAD 1.4826
 
+
+/*----------------------------------------------------------------------------*/
+/**
+  @hideinitializer
+  @brief  Return the minimum of two values
+  @param  first   The first expression in the comparison
+  @param  second  The second expression in the comparison
+  @return The minimum of the two values
+  @note If the first argument is the smallest then it is evaluated twice
+        otherwise the second argument is evaluated twice
+
+ */
+/*----------------------------------------------------------------------------*/
+#define CPL_MIN(first, second)                                          \
+    ((first) < (second) ? (first) : (second)) /* Two evaluations of minimum */
+
+/*----------------------------------------------------------------------------*/
+/**
+  @hideinitializer
+  @brief  Return the maximum of two values
+  @param  first   The first expression in the comparison
+  @param  second  The second expression in the comparison
+  @return The maximum of the two values
+  @see CPL_MIN()
+
+ */
+/*----------------------------------------------------------------------------*/
+#define CPL_MAX(first, second)                  \
+    ((first) > (second) ? (first) : (second)) /* Two evaluations of maximum */
+
 /**@}*/
 
 CPL_END_DECLS
