@@ -302,7 +302,7 @@ _cpl_frameset_history_merge(cx_list *self, cx_list *other,
             cx_multimap_const_iterator node2 = cx_list_get(other, first2);
 
             if (compare(cx_multimap_get_value(frames, node1),
-                        cx_multimap_get_value(frames, node2)) < 0) {
+                        cx_multimap_get_value(frames, node2)) > 0) {
 
                 cx_list_iterator next = cx_list_next(other, first2);
 
